//
// ********************************************************************
// * License and Disclaimer                                           *
// *                                                                  *
// * The  Geant4 software  is  copyright of the Copyright Holders  of *
// * the Geant4 Collaboration.  It is provided  under  the terms  and *
// * conditions of the Geant4 Software License,  included in the file *
// * LICENSE and available at  http://cern.ch/geant4/license .  These *
// * include a list of copyright holders.                             *
// *                                                                  *
// * Neither the authors of this software system, nor their employing *
// * institutes,nor the agencies providing financial support for this *
// * work  make  any representation or  warranty, express or implied, *
// * regarding  this  software system or assume any liability for its *
// * use.  Please see the license in the file  LICENSE  and URL above *
// * for the full disclaimer and the limitation of liability.         *
// *                                                                  *
// * This  code  implementation is the result of  the  scientific and *
// * technical work of the GEANT4 collaboration.                      *
// * By using,  copying,  modifying or  distributing the software (or *
// * any work based  on the software)  you  agree  to acknowledge its *
// * use  in  resulting  scientific  publications,  and indicate your *
// * acceptance of all terms of the Geant4 Software license.          *
// ********************************************************************
//
// $Id: D1EventAction.cc 93886 2015-11-03 08:28:26Z gcosmo $
//
/// \file D1EventAction.cc
/// \brief Implementation of the B1EventAction class
#include "D1EventAction.hh"
#include "D1Analysis.hh"

#include "G4RunManager.hh"
#include "G4Event.hh"
#include "G4SDManager.hh"
#include "G4HCofThisEvent.hh"
#include "G4UnitsTable.hh"

#include "Randomize.hh"
#include <iomanip>


//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......
 
D1EventAction::D1EventAction()
: G4UserEventAction(),
   fSha1EdepHCID(-1),
   fSha2EdepHCID(-1),
   fSha1TrackLengthHCID(-1),
   fSha2TrackLengthHCID(-1)
{}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......
 
D1EventAction::~D1EventAction()
{;}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

G4THitsMap<G4double>* 
D1EventAction::GetHitsCollection(G4int hcID,
                                  const G4Event* event) const
{
  G4THitsMap<G4double>* hitsCollection 
    = static_cast<G4THitsMap<G4double>*>(
        event->GetHCofThisEvent()->GetHC(hcID));
  
  if ( ! hitsCollection ) {
    G4ExceptionDescription msg;
    msg << "Cannot access hitsCollection ID " << hcID; 
    G4Exception("D1EventAction::GetHitsCollection()",
      "MyCode0003", FatalException, msg);
  }         

  return hitsCollection;
}    
//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

G4double D1EventAction::GetSum(G4THitsMap<G4double>* hitsMap) const
{
  G4double sumValue = 0;
  std::map<G4int, G4double*>::iterator it;
  for ( it = hitsMap->GetMap()->begin(); it != hitsMap->GetMap()->end(); it++) {
    sumValue += *(it->second);
  }
  return sumValue;  
}  
//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

void D1EventAction::PrintEventStatistics(
                            G4double sha1Edep, G4double sha1TrackLength,
                         G4double sha2Edep, G4double sha2TrackLength) const
{
  // Print event statistics
  //
  G4cout
     << "   Shape1: total energy: " 
     << std::setw(7) << G4BestUnit(sha1Edep, "Energy")
    << "       total track length: " 
     << std::setw(7) << G4BestUnit(sha1TrackLength, "Length")
     << G4endl
     << "   Shape2: total energy: " 
     << std::setw(7) << G4BestUnit(sha2Edep, "Energy")
     << "       total track length: " 
     << std::setw(7) << G4BestUnit(sha2TrackLength, "Length")
     << G4endl;
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......
 
void D1EventAction::BeginOfEventAction(const G4Event*)
{}


//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......
 
void D1EventAction::EndOfEventAction(const G4Event* event)
{
// Get hist collections IDs
  if ( fSha1EdepHCID == -1 ) {
    fSha1EdepHCID 
      = G4SDManager::GetSDMpointer()->GetCollectionID("Shape1/Edep");
    fSha2EdepHCID 
      = G4SDManager::GetSDMpointer()->GetCollectionID("Shape2/Edep");
     fSha1TrackLengthHCID 
      = G4SDManager::GetSDMpointer()->GetCollectionID("Shape1/TrackLength");
    fSha2TrackLengthHCID 
      = G4SDManager::GetSDMpointer()->GetCollectionID("Shape2/TrackLength");


}

// Get sum values from hits collections
  //
  G4double sha1Edep = GetSum(GetHitsCollection(fSha1EdepHCID, event));
  G4double sha2Edep = GetSum(GetHitsCollection(fSha2EdepHCID, event));

  G4double sha1TrackLength 
    = GetSum(GetHitsCollection(fSha1TrackLengthHCID, event));
  G4double sha2TrackLength 
    = GetSum(GetHitsCollection(fSha2TrackLengthHCID, event));
  

// get analysis manager
  G4AnalysisManager* analysisManager = G4AnalysisManager::Instance();

// fill histograms
  //  
  analysisManager->FillH1(1, sha1Edep);
  analysisManager->FillH1(2, sha2Edep);
  analysisManager->FillH1(3, sha1TrackLength);
  analysisManager->FillH1(4, sha2TrackLength);


// fill ntuple
  //
  analysisManager->FillNtupleDColumn(0, sha1Edep);
  analysisManager->FillNtupleDColumn(1, sha2Edep);
  analysisManager->FillNtupleDColumn(2, sha1TrackLength);
  analysisManager->FillNtupleDColumn(3, sha2TrackLength);
  analysisManager->AddNtupleRow();  

 //print per event (modulo n)
  //
  G4int eventID = event->GetEventID();
  G4int printModulo = G4RunManager::GetRunManager()->GetPrintProgress();
  if ( ( printModulo > 0 ) && ( eventID % printModulo == 0 ) ) {
    G4cout << "---> End of event: " << eventID << G4endl;     
    PrintEventStatistics(sha1Edep, sha2Edep, sha1TrackLength, sha2TrackLength);
  }
}



